@@ -116,9 +116,9 @@ static ip_type4 ip_from_internal_list(char* name, size_t len) {
 
 	internal_ips->list[internal_ips->counter] = new_mem;
 	internal_ips->list[internal_ips->counter]->hash = hash;
-	
+
 	new_mem = dumpstring((char*) name, len + 1);
-	
+
 	if(!new_mem) {
 		internal_ips->list[internal_ips->counter] = 0;
 		goto oom;
@@ -131,7 +131,7 @@ static ip_type4 ip_from_internal_list(char* name, size_t len) {
 
 	return res;
 	err_plus_unlock:
-	
+
 	PDEBUG("return err\n");
 	return ip_type_invalid.addr.v4;
 }
@@ -244,7 +244,7 @@ static void* threadfunc(void* x) {
 	(void) x;
 	int ret;
 	struct at_msghdr msg;
-	union { 
+	union {
 		char host[MSG_LEN_MAX];
 		ip_type4 ip;
 	} readbuf;
@@ -309,7 +309,18 @@ size_t at_get_host_for_ip(ip_type4 ip, char* readbuf) {
 
 
 static void initpipe(int* fds) {
-	if(pipe(fds) == -1) {
+	int retval;
+
+#ifdef HAVE_PIPE2
+	retval = pipe2(fds, O_CLOEXEC);
+#else
+	retval = pipe(fds);
+	if(retval == 0) {
+		fcntl(fds[0], F_SETFD, FD_CLOEXEC);
+		fcntl(fds[1], F_SETFD, FD_CLOEXEC);
+	}
+#endif
+	if(retval == -1) {
 		perror("pipe");
 		exit(1);
 	}
